# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType

from ._client_factory import (cf_cdn, cf_custom_domain, cf_endpoints, cf_profiles, cf_origins, cf_resource_usage,
                              cf_edge_nodes)


def load_command_table(self, _):

    def _not_found(message):
        def _inner_not_found(ex):
            from azure.mgmt.cdn.models import ErrorResponseException
            from knack.util import CLIError

            if isinstance(ex, ErrorResponseException) \
                    and ex.response is not None \
                    and ex.response.status_code == 404:
                raise CLIError(message)
            raise ex
        return _inner_not_found

    not_found_msg = "{}(s) not found. Please verify the resource(s), group or it's parent resources " \
                    "exist."
    profile_not_found_msg = not_found_msg.format('Profile')
    endpoint_not_found_msg = not_found_msg.format('Endpoint')
    cd_not_found_msg = not_found_msg.format('Custom Domain')
    origin_not_found_msg = not_found_msg.format('Origin')

    cdn_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.cdn#CdnManagementClient.{}',
        client_factory=cf_cdn
    )

    cdn_endpoints_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.cdn.operations.endpoints_operations#EndpointsOperations.{}',
        client_factory=cf_endpoints,
        exception_handler=_not_found(endpoint_not_found_msg)
    )

    cdn_profiles_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.cdn.operations.profiles_operations#ProfilesOperations.{}',
        client_factory=cf_profiles,
        exception_handler=_not_found(profile_not_found_msg)
    )

    cdn_domain_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.cdn.operations.custom_domains_operations#CustomDomainsOperations.{}',
        client_factory=cf_custom_domain,
        exception_handler=_not_found(cd_not_found_msg)
    )

    cdn_origin_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.cdn.operations.origins_operations#OriginsOperations.{}',
        client_factory=cf_origins,
        exception_handler=_not_found(origin_not_found_msg)
    )

    cdn_edge_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.cdn.operations.edge_nodes_operations#EdgeNodesOperations.{}',
        client_factory=cf_edge_nodes
    )

    cdn_usage_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.cdn.operations.resource_usage_operations#ResourceUsageOperations.{}',
        client_factory=cf_resource_usage
    )

    with self.command_group('cdn', cdn_sdk) as g:
        g.command('name-exists', 'check_name_availability')

    with self.command_group('cdn', cdn_usage_sdk) as g:
        g.command('usage', 'list')

    with self.command_group('cdn endpoint', cdn_endpoints_sdk) as g:
        for name in ['start', 'stop', 'delete']:
            g.command(name, name)
        g.show_command('show', 'get')
        g.command('list', 'list_by_profile')
        g.command('load', 'load_content')
        g.command('purge', 'purge_content')
        g.command('validate-custom-domain', 'validate_custom_domain')
        g.custom_command('create', 'create_endpoint', client_factory=cf_cdn,
                         doc_string_source='azure.mgmt.cdn.models#Endpoint')
        g.generic_update_command('update', setter_name='update', setter_arg_name='endpoint_update_properties',
                                 custom_func_name='update_endpoint',
                                 doc_string_source='azure.mgmt.cdn.models#EndpointUpdateParameters')

    with self.command_group('cdn profile', cdn_profiles_sdk) as g:
        g.show_command('show', 'get')
        g.command('usage', 'list_resource_usage')
        g.command('delete', 'delete')
        g.custom_command('list', 'list_profiles', client_factory=cf_cdn)
        g.custom_command('create', 'create_profile', client_factory=cf_cdn)
        g.generic_update_command('update', setter_name='update', custom_func_name='update_profile',
                                 doc_string_source='azure.mgmt.cdn.models#ProfileUpdateParameters')

    with self.command_group('cdn custom-domain', cdn_domain_sdk) as g:
        g.show_command('show', 'get')
        g.command('delete', 'delete')
        g.command('list', 'list_by_endpoint')
        g.custom_command('create', 'create_custom_domain', client_factory=cf_cdn)
        g.command('enable-https', 'enable_custom_https')
        g.command('disable-https', 'disable_custom_https')

    with self.command_group('cdn origin', cdn_origin_sdk) as g:
        g.show_command('show', 'get')
        g.command('list', 'list_by_endpoint')

    with self.command_group('cdn edge-node', cdn_edge_sdk) as g:
        g.command('list', 'list')



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps


helps['cdn'] = """
    type: group
    short-summary: Manage Azure Content Delivery Networks (CDNs).
"""

helps['cdn profile'] = """
    type: group
    short-summary: Manage CDN profiles to define an edge network.
"""

helps['cdn profile create'] = """
    type: command
    short-summary: Create a new CDN profile.
    parameters:
        - name: --sku
          type: string
          short-summary: >
            The pricing tier (defines a CDN provider, feature list and rate) of the CDN profile.
            Defaults to Standard_Akamai.
    examples:
        - name: Create a CDN profile using Verizon premium CDN.
          text: >
            az cdn profile create -g group -n profile --sku Premium_Verizon
"""

helps['cdn profile update'] = """
    type: command
    short-summary: Update a CDN profile.
"""

helps['cdn profile delete'] = """
    type: command
    short-summary: Delete a CDN profile.
    examples:
        - name: Delete a CDN profile.
          text: >
            az cdn profile delete -g group -n profile
"""

helps['cdn profile list'] = """
    type: command
    short-summary: List CDN profiles.
    examples:
        - name: List CDN profiles in a resource group.
          text: >
            az cdn profile list -g group
"""

helps['cdn endpoint'] = """
    type: group
    short-summary: Manage CDN endpoints.
"""

helps['cdn endpoint create'] = """
    type: command
    short-summary: Create a named endpoint to connect to a CDN.
    examples:
        - name: Create an endpoint to service content for hostname over HTTP or HTTPS.
          text: >
            az cdn endpoint create -g group -n endpoint --profile-name profile \\
                --origin www.example.com
        - name: Create an endpoint with a custom domain origin with HTTP and HTTPS ports.
          text: >
            az cdn endpoint create -g group -n endpoint --profile-name profile \\
                --origin www.example.com 88 4444
        - name: Create an endpoint with a custom domain with compression and only HTTPS.
          text: >
            az cdn endpoint create -g group -n endpoint --profile-name profile \\
                --origin www.example.com --no-http --enable-compression
"""

helps['cdn endpoint update'] = """
    type: command
    short-summary: Update a CDN endpoint to manage how content is delivered.
    examples:
        - name: Turn off HTTP traffic for an endpoint.
          text: >
            az cdn endpoint update -g group -n endpoint --profile-name profile --no-http
        - name: Enable content compression for an endpoint.
          text: >
            az cdn endpoint update -g group -n endpoint --profile-name profile \\
                --enable-compression
"""


helps['cdn endpoint delete'] = """
    type: command
    short-summary: Delete a CDN endpoint.
    examples:
        - name: Delete a CDN endpoint.
          text: >
            az cdn endpoint delete -g group -n endpoint --profile-name profile-name
"""

helps['cdn endpoint start'] = """
    type: command
    short-summary: Start a CDN endpoint.
    examples:
        - name: Start a CDN endpoint.
          text: >
            az cdn endpoint start -g group -n endpoint --profile-name profile-name
"""

helps['cdn endpoint stop'] = """
    type: command
    short-summary: Stop a CDN endpoint.
    examples:
        - name: Stop a CDN endpoint.
          text: >
            az cdn endpoint stop -g group -n endpoint --profile-name profile-name
"""

helps['cdn endpoint load'] = """
    type: command
    short-summary: Pre-load content for a CDN endpoint.
    examples:
        - name: Pre-load Javascript and CSS content for an endpoint.
          text: >
            az cdn endpoint load -g group -n endpoint --profile-name profile-name --content-paths \\
                '/scripts/app.js' '/styles/main.css'
"""

helps['cdn endpoint purge'] = """
    type: command
    short-summary: Purge pre-loaded content for a CDN endpoint.
    examples:
        - name: Purge pre-loaded Javascript and CSS content.
          text: >
            az cdn endpoint purge -g group -n endpoint --profile-name profile-name --content-paths \\
                '/scripts/app.js' '/styles/*'
"""

helps['cdn endpoint list'] = """
    type: command
    short-summary: List available endpoints for a CDN.
    examples:
        - name: List all endpoints within a given CDN profile.
          text: >
            az cdn endpoint list -g group --profile-name profile-name
"""

helps['cdn custom-domain'] = """
    type: group
    short-summary: Manage Azure CDN Custom Domains to provide custom host names for endpoints.
"""

helps['cdn custom-domain delete'] = """
    type: command
    short-summary: Delete the custom domain of a CDN.
    examples:
        - name: Delete a custom domain.
          text: >
            az cdn custom-domain delete -g group --endpoint-name endpoint --profile-name profile \\
                -n domain-name
"""

helps['cdn custom-domain show'] = """
    type: command
    short-summary: Show details for the custom domain of a CDN.
    examples:
        - name: Get the details of a custom domain.
          text: >
            az cdn custom-domain show -g group --endpoint-name endpoint --profile-name profile \\
                -n domain-name
"""

helps['cdn custom-domain create'] = """
    type: command
    short-summary: Create a new custom domain to provide a hostname for a CDN endpoint.
    long-summary: >
        Creates a new custom domain which must point to the hostname of the endpoint.
        For example, the custom domain hostname cdn.contoso.com would need to have a
        CNAME record pointing to the hostname of the endpoint related to this custom
        domain.
    parameters:
        - name: --profile-name
          type: string
          short-summary: Name of the CDN profile which is unique within the resource group.
        - name: --endpoint-name
          type: string
          short-summary: Name of the endpoint under the profile which is unique globally.
        - name: --hostname
          type: string
          short-summary: The host name of the custom domain. Must be a domain name.
    examples:
        - name: Create a custom domain within an endpoint and profile.
          text: >
            az cdn custom-domain create -g group --endpoint-name endpoint --profile-name profile \\
                -n domain-name --hostname www.example.com
"""

helps['cdn origin'] = """
    type: group
    short-summary: List or show existing origins related to CDN endpoints.
"""

helps['cdn edge-node'] = """
    type: group
    short-summary: View all available CDN edge nodes.
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.mgmt.cdn.models import (Endpoint, QueryStringCachingBehavior, SkuName,
                                   EndpointUpdateParameters, ProfileUpdateParameters)


def default_content_types():
    return ["text/plain",
            "text/html",
            "text/css",
            "text/javascript",
            "application/x-javascript",
            "application/javascript",
            "application/json",
            "application/xml"]


def _update_mapper(existing, new, keys):
    for key in keys:
        existing_value = getattr(existing, key)
        new_value = getattr(new, key)
        setattr(new, key, new_value if new_value is not None else existing_value)


# region Custom Commands
# pylint: disable=unused-argument
def list_profiles(cmd, client, resource_group_name=None):
    profiles = client.profiles
    profile_list = profiles.list_by_resource_group(resource_group_name=resource_group_name) \
        if resource_group_name else profiles.list()
    return list(profile_list)


# pylint: disable=unused-argument
def update_endpoint(cmd, instance,
                    origin_host_header=None,
                    origin_path=None,
                    content_types_to_compress=None,
                    is_compression_enabled=None,
                    is_http_allowed=None,
                    is_https_allowed=None,
                    query_string_caching_behavior=None,
                    tags=None):
    params = EndpointUpdateParameters(
        origin_host_header=origin_host_header,
        origin_path=origin_path,
        content_types_to_compress=content_types_to_compress,
        is_compression_enabled=is_compression_enabled,
        is_http_allowed=is_http_allowed,
        is_https_allowed=is_https_allowed,
        query_string_caching_behavior=query_string_caching_behavior,
        tags=tags
    )

    if is_compression_enabled and not instance.content_types_to_compress:
        params.content_types_to_compress = default_content_types()

    _update_mapper(instance, params, [
        'origin_host_header',
        'origin_path',
        'content_types_to_compress',
        'is_compression_enabled',
        'is_http_allowed',
        'is_https_allowed',
        'query_string_caching_behavior',
        'tags'
    ])
    return params


# pylint: disable=unused-argument
def create_endpoint(cmd, client, resource_group_name, profile_name, name, origins, location=None,
                    origin_host_header=None, origin_path=None, content_types_to_compress=None,
                    is_compression_enabled=None, is_http_allowed=None, is_https_allowed=None,
                    query_string_caching_behavior=QueryStringCachingBehavior.ignore_query_string.
                    value, tags=None):
    is_compression_enabled = False if is_compression_enabled is None else is_compression_enabled
    is_http_allowed = True if is_http_allowed is None else is_http_allowed
    is_https_allowed = True if is_https_allowed is None else is_https_allowed
    endpoint = Endpoint(location=location,
                        origins=origins,
                        origin_host_header=origin_host_header,
                        origin_path=origin_path,
                        content_types_to_compress=content_types_to_compress,
                        is_compression_enabled=is_compression_enabled,
                        is_http_allowed=is_http_allowed,
                        is_https_allowed=is_https_allowed,
                        query_string_caching_behavior=query_string_caching_behavior,
                        tags=tags)
    if is_compression_enabled and not endpoint.content_types_to_compress:
        endpoint.content_types_to_compress = default_content_types()

    return client.endpoints.create(resource_group_name, profile_name, name, endpoint)


# pylint: disable=unused-argument
def create_custom_domain(cmd, client, resource_group_name, profile_name, endpoint_name, custom_domain_name,
                         hostname, location=None, tags=None):
    return client.custom_domains.create(resource_group_name,
                                        profile_name,
                                        endpoint_name,
                                        custom_domain_name,
                                        hostname)


# pylint: disable=unused-argument
def update_profile(cmd, instance, tags=None):
    params = ProfileUpdateParameters(tags=tags)
    _update_mapper(instance, params, ['tags'])
    return params


# pylint: disable=unused-argument
def create_profile(cmd, client, resource_group_name, name,
                   sku=SkuName.standard_akamai.value,
                   location=None, tags=None):
    from azure.mgmt.cdn.models import (Profile, Sku)
    profile = Profile(location=location, sku=Sku(name=sku), tags=tags)
    return client.profiles.create(resource_group_name, name, profile)

# endregion
